@@ -1,10 +1,10 @@
-my $RCS_Id = '$Id: Procmail.pm,v 1.19 2002-05-08 13:48:56+02 jv Exp $ ';
+my $RCS_Id = '$Id: Procmail.pm,v 1.24 2004-09-19 12:34:56+02 jv Exp jv $ ';
 
 # Author          : Johan Vromans
 # Created On      : Tue Aug  8 13:53:22 2000
 # Last Modified By: Johan Vromans
 # Last Modified On:
-# Update Count    : 229
+# Update Count    : 254
 # Status          : Unknown, Use with caution!
 
 =head1 NAME
@@ -56,8 +56,7 @@ in The Perl Journal #18, that set it off.
 
 I first started using Simon's great module, and then decided to write
 my own since I liked certain things to be done differently. And I
-couldn't wait for his updates. Currently, Simon and I are in the
-process of considering to port my enhancements to his code as well.
+couldn't wait for his updates.
 
 C<Mail::Procmail> allows a piece of email to be logged, examined,
 delivered into a mailbox, filtered, resent elsewhere, rejected, and so
@@ -137,7 +136,7 @@ take place.
 
 package Mail::Procmail;
 
-$VERSION = "1.03";
+$VERSION = "1.08";
 
 use strict;
 use 5.005;
@@ -166,12 +165,15 @@ require Exporter;
 @EXPORT = qw(
 	     pm_init
 	     pm_gethdr
+	     pm_gethdr_raw
+	     pm_body
 	     pm_deliver
 	     pm_reject
 	     pm_resend
 	     pm_pipe_to
 	     pm_command
 	     pm_ignore
+	     pm_dupcheck
 	     pm_lockfile
 	     pm_unlockfile
 	     pm_log
@@ -203,6 +205,12 @@ Attributes:
 
 =item *
 
+fh
+
+An open file handle to read the message from. Defaults to STDIN.
+
+=item *
+
 logfile
 
 The name of a file to log messages to. Each message will have a timestamp
@@ -251,6 +259,7 @@ sub pm_init {
     my %atts = (
 		logfile   => '',
 		loglevel  => 0,
+		fh	  => undef,
 		verbose   => 0,
 		trace     => 0,
 		debug     => 0,
@@ -262,12 +271,14 @@ sub pm_init {
     $verbose   = delete $atts{verbose};
     $logfile   = delete $atts{logfile};
     $loglevel  = delete $atts{loglevel};
+    my $fh     = delete $atts{fh} || \*STDIN;
+
     $trace |= ($debug || $test);
 
     croak("Unprocessed attributes: ".join(" ",sort keys %atts))
       if %atts;
 
-    $m_obj = Mail::Internet->new(\*STDIN);
+    $m_obj = Mail::Internet->new($fh);
     $m_head = $m_obj->head;  # Mail::Header
 
     $m_obj;
@@ -310,6 +321,43 @@ sub pm_gethdr {
     wantarray ? @ret : '';
 }
 
+=head2 pm_gethdr_raw
+
+Like pm_gethdr, but without whitespace cleanup.
+
+=cut
+
+sub pm_gethdr_raw {
+    my ($hdr, $ix) = @_;
+    my @ret;
+    foreach my $val ( $m_head->get($hdr, $ix) ) {
+	last unless defined $val;
+	if ( $debug ) {
+	    $hdr =~ s/-(.)/"-".ucfirst($1)/ge;
+	    warn (ucfirst($hdr), ": ", $val, "\n");
+	}
+	return $val unless wantarray;
+	push (@ret, $val);
+    }
+    wantarray ? @ret : '';
+}
+
+=head2 pm_body
+
+This routine fetches the body of a message, as a reference to an array
+of lines.
+
+Example:
+
+    $body = pm_body();			# ref of lines
+    $body = join("", @{pm_body()});	# as one string
+
+=cut
+
+sub pm_body {
+    $m_obj->body;
+}
+
 =head2 pm_deliver
 
 This routine performs delivery to a Unix style mbox file, or maildir.
@@ -337,10 +385,14 @@ program will exit with a DELIVERED status.
 
 =cut
 
+sub _pm_msg_size {
+    length($m_obj->head->as_string || '') + length(join("", @{$m_obj->body}));
+}
+
 sub pm_deliver {
     my ($target, %atts) = @_;
     my $line = (caller(0))[2];
-    pm_log(2, "deliver[$line]: $target");
+    pm_log(2, "deliver[$line]: $target "._pm_msg_size());
 
     # Is it a Maildir?
     if ( -d "$target/tmp" && -d "$target/new" ) {
@@ -439,7 +491,7 @@ Do this, even in test mode.
 sub pm_pipe_to {
     my ($target, %atts) = @_;
     my $line = (caller(0))[2];
-    pm_log(2, "pipe_to[$line]: $target");
+    pm_log(2, "pipe_to[$line]: $target "._pm_msg_size());
 
     my $lock;
     my $lockfile = $atts{lockfile};
@@ -502,7 +554,7 @@ Do this, even in test mode.
 sub pm_command {
     my ($target, %atts) = @_;
     my $line = (caller(0))[2];
-    pm_log(2, "command[$line]: $target");
+    pm_log(2, "command[$line]: $target "._pm_msg_size());
 
     my $lock;
     my $lockfile = $atts{lockfile};
@@ -542,7 +594,7 @@ program will exit with a DELIVERED status.
 sub pm_resend {
     my ($target, %atts) = @_;
     my $line = (caller(0))[2];
-    pm_log(2, "resend[$line]: $target");
+    pm_log(2, "resend[$line]: $target "._pm_msg_size());
     $m_obj->smtpsend(To => $target) unless $test;
     exit DELIVERED unless $atts{continue};
 }
@@ -561,7 +613,7 @@ Example:
 sub pm_reject {
     my $reason = shift;
     my $line = (caller(0))[2];
-    pm_log(2, "reject[$line]: $reason");
+    pm_log(2, "reject[$line]: $reason "._pm_msg_size());
     print STDERR ($reason, "\n") unless lc $logfile eq 'stderr';
     exit REJECTED;
 }
@@ -582,7 +634,7 @@ Example:
 sub pm_ignore {
     my $reason = shift;
     my $line = (caller(0))[2];
-    pm_log(2, "ignore[$line]: $reason");
+    pm_log(2, "ignore[$line]: $reason "._pm_msg_size());
     exit DELIVERED;
 }
 
@@ -593,7 +645,7 @@ already been received.
 
 Example:
 
-    pm_dupcheck(pm_gethdr("msg-id"));
+    pm_dupcheck(scalar(pm_gethdr("message-id")));
 
 Attributes:
 
@@ -643,7 +695,7 @@ sub pm_dupcheck {
 	if ( defined($tmp = $msgid{$msgid}) ) {
 	    if ( ($msgid{$msgid} = time) - $tmp < $atts{retain}*24*60*60 ) {
 		my $line = (caller(0))[2];
-		pm_log(2, "dup[$line]: $msgid");
+		pm_log(2, "dup[$line]: $msgid "._pm_msg_size());
 		$dup++;
 	    }
 	}
@@ -875,8 +927,15 @@ sendmail, or postfix).
     :0:
     $HOME/syslog/mail
 
-The original contents of the .procmailrc can be safely left in place
-after these lines.
+B<WARNING>: procmail seems to have problems when $HOME/syslog/mail
+gets too big (over 50Mb). If you want to maintain a huge archive, you
+can specify excess extents, like this:
+
+    :0:
+    $HOME/syslog/mail-ext1
+
+    :0:
+    $HOME/syslog/mail-ext2
 
 =head1 EXAMPLE
 
@@ -901,7 +960,7 @@ by Tom Christiansen.
 
 =head1 COPYRIGHT and DISCLAIMER
 
-This program is Copyright 2000 by Squirrel Consultancy. All
+This program is Copyright 2000,2004 by Squirrel Consultancy. All
 rights reserved.
 
 This program is free software; you can redistribute it and/or modify
